/**
 * Retrieves a CSS file for a given theme name.
 *
 * @param { string } themeName - The name of the theme to load
    * @returns { Promise < string | null >} The CSS content of the theme or null if not found
        *
 * @example
        * // Successful theme load
 * const darkTheme = await themes('dark');
 *
 * @example
    * // Theme not found
 * const missingTheme = await themes('nonexistent');
 * // Returns null
 */
export function themeChanger(themeName: string): Promise<string | null>;
export function currentTheme(): string;
export { default } from "./actions.svelte.js";
export { default as l10n } from "flatpickr_plus/dist/l10n";
/**
 * Enum-like object defining available theme names
 */
export type themeNames = string;
export namespace themeNames {
    let _default: string;
    export { _default as default };
    export let light: string;
    export let dark: string;
    export let materialBlue: string;
    export let materialGreen: string;
    export let materialRed: string;
    export let materialOrange: string;
    export let airbnb: string;
    export let confetti: string;
}
